/*
   Original Author: bbum

   Copyright 1999 by CodeFab, Inc.   All rights reserved.

   This notice may not be removed from this source code.
   The use and distribution of this software is governed by the
   terms of the MiscKit license agreement.  Refer to the license
   document included with the MiscKit distribution for the terms.
   Details of the license agreement may be found at
   www.misckit.com or within the MiscKit distribution.

   There may be more information about this software on
   www.codefab.com.  Send queries to ask_codefab@codefab.com.
*/

#import "ErrorValueComponent.h"

@interface WOSession (scripted)
- (void) appendError: (NSString *) aString;
@end

@implementation ErrorValueComponent
- init
{
    self = [super init];

    [self resetErrorValueDictionary];

    return self;
}

- (void) dealloc
{
    [errorValueDictionary release];
    errorValueDictionary = nil;

    [super dealloc]; 
}

- (void) resetErrorValueDictionary;
{	
    if (errorValueDictionary != nil) {
        [errorValueDictionary release];
        errorValueDictionary = nil;
    }

    errorValueDictionary = [NSMutableDictionary dictionary];
    [errorValueDictionary retain];
}

- (void)appendToResponse:(WOResponse *)aResponse inContext:(WOContext *)aContext;
{
    [super appendToResponse: aResponse inContext: aContext];
    [self resetErrorValueDictionary];
}

- (id)valueForKeyPath:(NSString *)aPath;
{
    id possibleValue = [errorValueDictionary objectForKey: aPath];
    if (possibleValue != nil) {
        if (possibleValue == [EONull null])  // see if there was a nil placeholder
            return nil;
        else
            return possibleValue;
    }

    return [super valueForKeyPath: aPath];
}

- (void)validationFailedWithException:(NSException *)exception value:(id)value keyPath:(NSString *)keyPath;
{
    [self logWithFormat: @"%@ -validationFailedWithException:value:keyPath:\n\tException: %@ / %@\n\tv: %@ -=> kP: %@", [self name], [exception name], [exception reason], value, keyPath];
    [[self session] appendError: [NSString stringWithFormat: @"(Key path: %@) Reason: %@", keyPath, [exception reason] ] ];
    if (value == nil)
        value = [EONull null]; // eonull as placeholder for nil;  just like EOClassDescription.h describes it.
    [errorValueDictionary setObject: value forKey: keyPath];
}
@end
